#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json


class ThriftJSONEncoder(json.JSONEncoder):

    def default(self, o):
        if not hasattr(o, 'thrift_spec'):
            return super(ThriftJSONEncoder, self).default(o)

        spec = getattr(o, 'thrift_spec')
        ret = {}
        for field in spec:
            if field is None:
                continue
            (tag, field_ttype, field_name, field_ttype_info, default) = field
            if field_name in o.__dict__:
                val = o.__dict__[field_name]
                if val != default:
                    ret[field_name] = val
        return ret


def thrift2json(obj):
    return json.dumps(obj, cls=ThriftJSONEncoder)


def thrift2dict(obj):
    str = thrift2json(obj)
    return json.loads(str)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

from thrift.Thrift import TType


class ThriftJSONDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        self._thrift_class = kwargs.pop('thrift_class')
        super(ThriftJSONDecoder, self).__init__(*args, **kwargs)

    def decode(self, json_str):
        if isinstance(json_str, dict):
            dct = json_str
        else:
            dct = super(ThriftJSONDecoder, self).decode(json_str)
        return self._convert(dct, TType.STRUCT,
                             (self._thrift_class, self._thrift_class.thrift_spec))

    def _convert(self, val, ttype, ttype_info):
        if ttype == TType.STRUCT:
            (thrift_class, thrift_spec) = ttype_info
            ret = thrift_class()
            for field in thrift_spec:
                if field is None:
                    continue
                (tag, field_ttype, field_name, field_ttype_info, dummy) = field
                if field_name not in val:
                    continue
                converted_val = self._convert(val[field_name], field_ttype, field_ttype_info)
                setattr(ret, field_name, converted_val)
        elif ttype == TType.LIST:
            (element_ttype, element_ttype_info) = ttype_info
            ret = [self._convert(x, element_ttype, element_ttype_info) for x in val]
        elif ttype == TType.SET:
            (element_ttype, element_ttype_info) = ttype_info
            ret = set([self._convert(x, element_ttype, element_ttype_info) for x in val])
        elif ttype == TType.MAP:
            (key_ttype, key_ttype_info, val_ttype, val_ttype_info) = ttype_info
            ret = dict([(self._convert(k, key_ttype, key_ttype_info),
                         self._convert(v, val_ttype, val_ttype_info)) for (k, v) in val.iteritems()])
        elif ttype == TType.STRING:
            if isinstance(val, unicode):
                ret = val.encode("utf8")
            else:
                ret = str(val)
        elif ttype == TType.DOUBLE:
            ret = float(val)
        elif ttype == TType.I64:
            ret = long(val)
        elif ttype == TType.I32 or ttype == TType.I16 or ttype == TType.BYTE:
            ret = int(val)
        elif ttype == TType.BOOL:
            ret = bool(val)
        else:
            raise TypeError('Unrecognized thrift field type: %d' % ttype)
        return ret


def json2thrift(json_str, thrift_class):
    return json.loads(json_str, cls=ThriftJSONDecoder, thrift_class=thrift_class)

dict2thrift = json2thrift



#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class Filter:
  """
  Attributes:
   - attr
   - op
   - val
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'attr', None, None, ), # 1
    (2, TType.STRING, 'op', None, None, ), # 2
    (3, TType.STRING, 'val', None, None, ), # 3
  )

  def __init__(self, attr=None, op=None, val=None,):
    self.attr = attr
    self.op = op
    self.val = val

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.attr = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.op = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.val = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Filter')
    if self.attr is not None:
      oprot.writeFieldBegin('attr', TType.STRING, 1)
      oprot.writeString(self.attr)
      oprot.writeFieldEnd()
    if self.op is not None:
      oprot.writeFieldBegin('op', TType.STRING, 2)
      oprot.writeString(self.op)
      oprot.writeFieldEnd()
    if self.val is not None:
      oprot.writeFieldBegin('val', TType.STRING, 3)
      oprot.writeString(self.val)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.attr is None:
      raise TProtocol.TProtocolException(message='Required field attr is unset!')
    if self.op is None:
      raise TProtocol.TProtocolException(message='Required field op is unset!')
    if self.val is None:
      raise TProtocol.TProtocolException(message='Required field val is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.attr)
    value = (value * 31) ^ hash(self.op)
    value = (value * 31) ^ hash(self.val)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Flag:
  """
  Attributes:
   - name
   - weight
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'name', None, None, ), # 1
    (2, TType.I32, 'weight', None, None, ), # 2
  )

  def __init__(self, name=None, weight=None,):
    self.name = name
    self.weight = weight

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.name = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.weight = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Flag')
    if self.name is not None:
      oprot.writeFieldBegin('name', TType.STRING, 1)
      oprot.writeString(self.name)
      oprot.writeFieldEnd()
    if self.weight is not None:
      oprot.writeFieldBegin('weight', TType.I32, 2)
      oprot.writeI32(self.weight)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.name is None:
      raise TProtocol.TProtocolException(message='Required field name is unset!')
    if self.weight is None:
      raise TProtocol.TProtocolException(message='Required field weight is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.name)
    value = (value * 31) ^ hash(self.weight)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Branch:
  """
  Attributes:
   - path
   - volume
   - leaf
  """

  thrift_spec = (
    None, # 0
    (1, TType.LIST, 'path', (TType.STRUCT,(Filter, Filter.thrift_spec)), None, ), # 1
    (2, TType.I32, 'volume', None, None, ), # 2
    (3, TType.LIST, 'leaf', (TType.STRUCT,(Flag, Flag.thrift_spec)), None, ), # 3
  )

  def __init__(self, path=None, volume=None, leaf=None,):
    self.path = path
    self.volume = volume
    self.leaf = leaf

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.path = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = Filter()
            _elem5.read(iprot)
            self.path.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.volume = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.LIST:
          self.leaf = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = Flag()
            _elem11.read(iprot)
            self.leaf.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Branch')
    if self.path is not None:
      oprot.writeFieldBegin('path', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.path))
      for iter12 in self.path:
        iter12.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.volume is not None:
      oprot.writeFieldBegin('volume', TType.I32, 2)
      oprot.writeI32(self.volume)
      oprot.writeFieldEnd()
    if self.leaf is not None:
      oprot.writeFieldBegin('leaf', TType.LIST, 3)
      oprot.writeListBegin(TType.STRUCT, len(self.leaf))
      for iter13 in self.leaf:
        iter13.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.path is None:
      raise TProtocol.TProtocolException(message='Required field path is unset!')
    if self.volume is None:
      raise TProtocol.TProtocolException(message='Required field volume is unset!')
    if self.leaf is None:
      raise TProtocol.TProtocolException(message='Required field leaf is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.path)
    value = (value * 31) ^ hash(self.volume)
    value = (value * 31) ^ hash(self.leaf)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Feature:
  """
  Attributes:
   - name
   - key
   - on
   - expired_at
   - tree
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'name', None, None, ), # 1
    (2, TType.STRING, 'key', None, None, ), # 2
    (3, TType.BOOL, 'on', None, None, ), # 3
    (4, TType.I64, 'expired_at', None, None, ), # 4
    (5, TType.LIST, 'tree', (TType.STRUCT,(Branch, Branch.thrift_spec)), None, ), # 5
  )

  def __init__(self, name=None, key=None, on=None, expired_at=None, tree=None,):
    self.name = name
    self.key = key
    self.on = on
    self.expired_at = expired_at
    self.tree = tree

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.name = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.key = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.BOOL:
          self.on = iprot.readBool()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I64:
          self.expired_at = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.tree = []
          (_etype17, _size14) = iprot.readListBegin()
          for _i18 in xrange(_size14):
            _elem19 = Branch()
            _elem19.read(iprot)
            self.tree.append(_elem19)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Feature')
    if self.name is not None:
      oprot.writeFieldBegin('name', TType.STRING, 1)
      oprot.writeString(self.name)
      oprot.writeFieldEnd()
    if self.key is not None:
      oprot.writeFieldBegin('key', TType.STRING, 2)
      oprot.writeString(self.key)
      oprot.writeFieldEnd()
    if self.on is not None:
      oprot.writeFieldBegin('on', TType.BOOL, 3)
      oprot.writeBool(self.on)
      oprot.writeFieldEnd()
    if self.expired_at is not None:
      oprot.writeFieldBegin('expired_at', TType.I64, 4)
      oprot.writeI64(self.expired_at)
      oprot.writeFieldEnd()
    if self.tree is not None:
      oprot.writeFieldBegin('tree', TType.LIST, 5)
      oprot.writeListBegin(TType.STRUCT, len(self.tree))
      for iter20 in self.tree:
        iter20.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.name is None:
      raise TProtocol.TProtocolException(message='Required field name is unset!')
    if self.key is None:
      raise TProtocol.TProtocolException(message='Required field key is unset!')
    if self.on is None:
      raise TProtocol.TProtocolException(message='Required field on is unset!')
    if self.expired_at is None:
      raise TProtocol.TProtocolException(message='Required field expired_at is unset!')
    if self.tree is None:
      raise TProtocol.TProtocolException(message='Required field tree is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.name)
    value = (value * 31) ^ hash(self.key)
    value = (value * 31) ^ hash(self.on)
    value = (value * 31) ^ hash(self.expired_at)
    value = (value * 31) ^ hash(self.tree)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_thrift_json
----------------------------------

Tests for `thrift_json` module.
"""
import json
import unittest

from thrift_json import json2thrift, dict2thrift, thrift2json, thrift2dict

from .features import ttypes


class TestThriftJSON(unittest.TestCase):

    def test_to_thrift(self):
        expired_at = 100000
        feature = ttypes.Feature()
        feature.name = "new search"
        feature.key = "new_search"
        feature.expired_at = expired_at
        feature.on = True
        path1 = [ttypes.Filter("device.version", "==", json.dumps("ios9.0")),
                 ttypes.Filter("device.city", "in", json.dumps(["北京", "厦门"]))]
        branch1 = ttypes.Branch()
        branch1.path = path1
        branch1.volume = 100
        leaf1 = [ttypes.Flag("versionA", 20), ttypes.Flag("versionB", 40)]
        branch1.leaf = leaf1
        feature.tree = [branch1]

        feature_dct = {
            'name': 'new search',
            'key': 'new_search',
            'on': True,
            'expired_at': expired_at,
            'tree': [
                {
                    'path': [
                        {'attr': 'device.version', 'op': '==', 'val': json.dumps("ios9.0")},
                        {'attr': 'device.city', 'op': 'in', 'val': json.dumps(['北京', '厦门'])},
                    ],
                    'volume': 100,
                    'leaf': [
                        {'name': 'versionA', 'weight': 20},
                        {'name': 'versionB', 'weight': 40},
                    ]

                }
            ]
        }
        feature1 = dict2thrift(feature_dct, ttypes.Feature)
        assert feature1 == feature
        feature2 = json2thrift(json.dumps(feature_dct), ttypes.Feature)
        assert feature2 == feature1

    def test_from_thrift(self):
        expired_at = 100000
        feature = ttypes.Feature()
        feature.name = "new search"
        feature.key = "new_search"
        feature.expired_at = expired_at
        feature.on = True
        path1 = [ttypes.Filter("device.version", "==", json.dumps("ios9.0")),
                 ttypes.Filter("device.city", "in", json.dumps(["北京", "厦门"]))]
        branch1 = ttypes.Branch()
        branch1.path = path1
        branch1.volume = 100
        leaf1 = [ttypes.Flag("versionA", 20), ttypes.Flag("versionB", 40)]
        branch1.leaf = leaf1
        feature.tree = [branch1]
        feature_dct = thrift2dict(feature)
        assert feature_dct['key'] == feature.key



# -*- coding: utf-8 -*-
from .parser import ThriftJSONDecoder, json2thrift, dict2thrift
from .formatter import ThriftJSONEncoder, thrift2json, thrift2dict

__author__ = 'Young King'
__email__ = 'yanckin@gmail.com'
__version__ = '0.1.0'

__all__ = [
    'ThriftJSONDecoder', 'ThriftJSONEncoder',
    'json2thrift', 'dict2thrift',
    'thrift2json', 'thrift2dict'
]
